#!/usr/bin/python

"""Check for a copy of /branches/upstream to /tags, and generate an upstream
tarball in /tarballs
"""

import os
import re
import sys
import popen2
import shutil
import tempfile

REPO, REV = sys.argv[1:]
REPO = os.path.abspath(REPO)

tag_re = re.compile(r'^A   (?P<tag>tags/(?P<version>[^/]+))/$')

def run(cmd):
    """Run cmd in a shell

    If exit status is not 0, print a message and quit.
    """
    st = os.system(cmd)
    if st != 0:
        sys.stderr.write("Error running cmd:\n %s\n" % cmd)
        cleanup()
        sys.exit(1)

def cleanup():
    """Remove the temporary directory, if it's been created"""
    global tmpdir
    try:
        shutil.rmtree(tmpdir)
    except NameError:
        pass

def main():
    output, input = popen2.popen2('svnlook changed %s -r %s'
                                  % (REPO, REV))
    input.close()
    tags = []
    for line in output:
        m = tag_re.match(line)
        if m:
            tags.append((m.group('tag'), m.group('version')))
    output.close()

    upstream_tags = []
    for tag, version in tags:
        output, input = popen2.popen2('svnlook history %s %s' % (REPO, tag))
        lines = output.readlines()
        # Third line contains the current path, fourth line has the previous
        # path.  It should be /branches/upstream
        rev, path = lines[3].split()
        if path == '/branches/upstream':
            upstream_tags.append((tag, version))

    if upstream_tags:
        # Make a temporary directory and check out the tarballs directory into
        # it.  Global so that cleanup() can remove it without needing an arg
        global tmpdir
        tmpdir = tempfile.mkdtemp()

        cmd = 'svn co file://%s/tarballs %s/tarballs' % (REPO, tmpdir)
        run(cmd)

        for tag, version in upstream_tags:
            cmd = 'svn export file://%s/%s %s/piuparts-%s' % (REPO, tag,
                                                              tmpdir, version)
            run(cmd)
            
            cmd = 'tar zcf %s/tarballs/piuparts_%s.orig.tar.gz ' \
                  '-C %s piuparts-%s' % (tmpdir, version, tmpdir, version)
            run(cmd)

            cmd = 'svn add %s/tarballs/piuparts_%s.orig.tar.gz' % (tmpdir,
                                                                   version)
            run(cmd)

            cmd = 'svn commit %s/tarballs -m "Add upstream tarball for ' \
                  'version %s"' % (tmpdir, version)
            run(cmd)

        cleanup()


if __name__ == '__main__':
    main()
